searchNodes=[{"doc":"Bird View Finitomata provides a boilerplate for FSM implementation, allowing to concentrate on the business logic rather than on the process management and transitions/events consistency tweaking. It reads a description of the FSM from a string in PlantUML , Mermaid , or even custom format. Basically, it looks more or less like this PlantUML [ * ] -- &gt; s1 : to_s1 s1 -- &gt; s2 : to_s2 s1 -- &gt; s3 : to_s3 s2 -- &gt; [ * ] : ok s3 -- &gt; [ * ] : ok Mermaid s1 -- &gt; | to_s2 | s2 s1 -- &gt; | to_s3 | s3 It validates the FSM is consistent, namely it has a single initial state, one or more final states, and no orphan states. If everything is OK, it generates a GenServer that could be used both alone, and with provided supervision tree. This GenServer requires to implement three callbacks on_transition/4 — mandatory on_failure/3 — optional on_terminate/1 — optional All the callbacks do have a default implementation, that would perfectly handle transitions having a single to state and not requiring any additional business logic attached. Upon start, it moves to the next to initial state and sits there awaiting for the transition request . Then it would call an on_transition/4 callback and move to the next state, or remain in the current one, according to the response. Upon reachiung a final state, it would terminate itself. The process keeps all the history of states it went through, and might have a payload in its state. Example Let’s define the FSM instance defmodule MyFSM do @fsm &quot;&quot;&quot; [*] --&gt; s1 : to_s1 s1 --&gt; s2 : to_s2 s1 --&gt; s3 : to_s3 s2 --&gt; [*] : ok s3 --&gt; [*] : ok &quot;&quot;&quot; use Finitomata , @fsm def on_transition ( :s1 , :to_s2 , event_payload , state_payload ) , do : { :ok , :s2 , state_payload } end Now we can play with it a bit. children = [ Finitomata . child_spec ( ) ] Supervisor . start_link ( children , strategy : :one_for_one ) Finitomata . start_fsm MyFSM , &quot;My first FSM&quot; , %{ foo : :bar } Finitomata . transition &quot;My first FSM&quot; , { :to_s2 , nil } Finitomata . state &quot;My first FSM&quot; #⇒ %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}} Finitomata . allowed? &quot;My first FSM&quot; , :* # state #⇒ true Finitomata . responds? &quot;My first FSM&quot; , :to_s2 # event #⇒ false Finitomata . transition &quot;My first FSM&quot; , { :ok , nil } # to final state #⇒ [info] [◉ ⇄] [state: %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}}] Finitomata . alive? &quot;My first FSM&quot; #⇒ false Typically, one would implement all the on_transition/4 handlers, pattern matching on the state/event.","ref":"Finitomata.html","title":"Finitomata","type":"behaviour"},{"doc":"Returns true if the supervision tree is alive, false otherwise.","ref":"Finitomata.html#alive?/0","title":"Finitomata.alive?/0","type":"function"},{"doc":"Returns true if the FSM specified is alive, false otherwise.","ref":"Finitomata.html#alive?/1","title":"Finitomata.alive?/1","type":"function"},{"doc":"Returns true if the transition to the state state is possible, false otherwise.","ref":"Finitomata.html#allowed?/2","title":"Finitomata.allowed?/2","type":"function"},{"doc":"This callback will be called if the transition failed to complete to allow the consumer to take an action upon failure.","ref":"Finitomata.html#c:on_failure/3","title":"Finitomata.on_failure/3","type":"callback"},{"doc":"This callback will be called on transition to the final state to allow the consumer to perform some cleanup, or like.","ref":"Finitomata.html#c:on_terminate/1","title":"Finitomata.on_terminate/1","type":"callback"},{"doc":"This callback will be called from each transition processor.","ref":"Finitomata.html#c:on_transition/4","title":"Finitomata.on_transition/4","type":"callback"},{"doc":"Returns true if the transition by the event event is possible, false otherwise.","ref":"Finitomata.html#responds?/2","title":"Finitomata.responds?/2","type":"function"},{"doc":"Starts the FSM instance. The arguments are the implementation of FSM (the module, having use Finitomata ) the name of the FSM (might be any term, but it must be unique) the payload to be carried in the FSM state during the lifecycle The FSM is started supervised.","ref":"Finitomata.html#start_fsm/3","title":"Finitomata.start_fsm/3","type":"function"},{"doc":"The state of the FSM.","ref":"Finitomata.html#state/1","title":"Finitomata.state/1","type":"function"},{"doc":"Initiates the transition. The arguments are the name of the FSM {event, event_payload} tuple; the payload will be passed to the respective on_transition/4 call","ref":"Finitomata.html#transition/2","title":"Finitomata.transition/2","type":"function"},{"doc":"The payload that can be passed to each call to transition/3","ref":"Finitomata.html#t:event_payload/0","title":"Finitomata.event_payload/0","type":"type"},{"doc":"Error types of FSM validation","ref":"Finitomata.html#t:validation_error/0","title":"Finitomata.validation_error/0","type":"type"},{"doc":"Carries the state of the FSM.","ref":"Finitomata.State.html","title":"Finitomata.State","type":"module"},{"doc":"The payload that has been passed to the FSM instance on startup","ref":"Finitomata.State.html#t:payload/0","title":"Finitomata.State.payload/0","type":"type"},{"doc":"The internal representation of the FSM state","ref":"Finitomata.State.html#t:t/0","title":"Finitomata.State.t/0","type":"type"},{"doc":"The FSM boilerplate based on callbacks","ref":"readme.html","title":"Finitomata","type":"extras"},{"doc":"Bird View Finitomata provides a boilerplate for FSM implementation, allowing to concentrate on the business logic rather than on the process management and transitions/events consistency tweaking. It reads a description of the FSM from a string in PlantUML , Mermaid , or even custom format. Basically, it looks more or less like this PlantUML [ * ] -- &gt; s1 : to_s1 s1 -- &gt; s2 : to_s2 s1 -- &gt; s3 : to_s3 s2 -- &gt; [ * ] : ok s3 -- &gt; [ * ] : ok Mermaid s1 -- &gt; | to_s2 | s2 s1 -- &gt; | to_s3 | s3 It validates the FSM is consistent, namely it has a single initial state, one or more final states, and no orphan states. If everything is OK, it generates a GenServer that could be used both alone, and with provided supervision tree. This GenServer requires to implement three callbacks on_transition/4 — mandatory on_failure/3 — optional on_terminate/1 — optional All the callbacks do have a default implementation, that would perfectly handle transitions having a single to state and not requiring any additional business logic attached. Upon start, it moves to the next to initial state and sits there awaiting for the transition request . Then it would call an on_transition/4 callback and move to the next state, or remain in the current one, according to the response. Upon reachiung a final state, it would terminate itself. The process keeps all the history of states it went through, and might have a payload in its state. Example Let’s define the FSM instance defmodule MyFSM do @fsm &quot;&quot;&quot; [*] --&gt; s1 : to_s1 s1 --&gt; s2 : to_s2 s1 --&gt; s3 : to_s3 s2 --&gt; [*] : ok s3 --&gt; [*] : ok &quot;&quot;&quot; use Finitomata , @fsm def on_transition ( :s1 , :to_s2 , event_payload , state_payload ) , do : { :ok , :s2 , state_payload } end Now we can play with it a bit. children = [ Finitomata . child_spec ( ) ] Supervisor . start_link ( children , strategy : :one_for_one ) Finitomata . start_fsm MyFSM , &quot;My first FSM&quot; , %{ foo : :bar } Finitomata . transition &quot;My first FSM&quot; , { :to_s2 , nil } Finitomata . state &quot;My first FSM&quot; #⇒ %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}} Finitomata . allowed? &quot;My first FSM&quot; , :* # state #⇒ true Finitomata . responds? &quot;My first FSM&quot; , :to_s2 # event #⇒ false Finitomata . transition &quot;My first FSM&quot; , { :ok , nil } # to final state #⇒ [info] [◉ ⇄] [state: %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}}] Finitomata . alive? &quot;My first FSM&quot; #⇒ false Typically, one would implement all the on_transition/4 handlers, pattern matching on the state/event.","ref":"readme.html#introduction","title":"Finitomata - Introduction","type":"extras"},{"doc":"def deps do [ { :finitomata , &quot;~&gt; 0.1&quot; } ] end","ref":"readme.html#installation","title":"Finitomata - Installation","type":"extras"},{"doc":"0.2.0 — Mermaid support Documentation .","ref":"readme.html#changelog","title":"Finitomata - Changelog","type":"extras"}]